#![allow(dead_code)]
use std::fmt::Debug;

use otvec::{transform, Operation};

pub fn test_transform<T: PartialEq + Clone + Debug>(
    a: &Operation<T>,
    b: &Operation<T>,
    input: &[T],
    expect: &[T],
) {
    let (mut a2, mut b2) = (a.clone(), b.clone());
    transform(&mut a2, &mut b2);
    dbg!(&a2, &b2);
    check_op(&a2);
    check_op(&b2);

    let mut data = input.to_vec();
    a.clone().apply(&mut data);
    b2.apply(&mut data);
    assert_eq!(data, expect, "a, b2");

    let mut data = input.to_vec();
    b.clone().apply(&mut data);
    a2.apply(&mut data);
    assert_eq!(data, expect, "b, a2");
}

pub fn test_transform_sym<T: PartialEq + Clone + Debug>(
    a: &Operation<T>,
    b: &Operation<T>,
    input: &[T],
    expect: &[T],
) {
    test_transform(a, b, input, expect);
    test_transform(b, a, input, expect);
}

pub fn test_transform_cond<T: PartialEq + Clone + Debug>(
    a: &Operation<T>,
    b: &Operation<T>,
    input: &[T],
) {
    let (mut a2, mut b2) = (a.clone(), b.clone());
    transform(&mut a2, &mut b2);
    // dbg!(&a2, &b2);
    check_op(&a2);
    check_op(&b2);

    let mut data = input.to_vec();
    a.clone().apply(&mut data);
    b2.apply(&mut data);

    let mut data2 = input.to_vec();
    b.clone().apply(&mut data2);
    a2.apply(&mut data2);
    assert_eq!(data, data2);
}

fn check_op<T: Debug>(op: &Operation<T>) {
    match op {
        Operation::Nop => {}
        Operation::Ins { val, .. } => assert!(val.len() > 0, "empty op: {op:?}"),
        Operation::Del { n, .. } => assert!(*n > 0, "empty op: {op:?}"),
        Operation::Mov { pos, to } => assert!(pos != to, "empty op: {op:?}"),
        Operation::Seq { ops } => ops.iter().for_each(check_op),
    }
}

pub fn transformed_mov<T>(pos: usize, n: usize, to: usize) -> Operation<T> {
    Operation::Seq {
        ops: if to > pos {
            (pos..pos + n)
                .rev()
                .map(|i| Operation::Mov {
                    pos: i,
                    to: i + (to - pos),
                })
                .collect()
        } else {
            (pos..pos + n)
                .map(|i| Operation::Mov {
                    pos: i,
                    to: i - (pos - to),
                })
                .collect()
        },
    }
}
